/***************************************************************************
*
*	File name : boot.c
*	Function  : Boot function
*	Revision  :
*		07/31/2007	Nariai		First revision
*
****************************************************************************/

// Include File
#include	"reg_801.h"

// Prototype
void Boot(void);
static void LoadDataSec(void);

// Extern
extern int main(void);


/***************************************************************************
	Boot

	description	: boot function
	argument	:
	return: 	:

	input		:
	output		:

	flag		:
	global		:
***************************************************************************/
void Boot(void){

	asm("xld.a	%r0,	0x000000	");

	// Claer IRAM
	asm("xld.a	%r1,	0x000040	");
	asm("xld.a	%r2,	0x001000	");
	asm("CLEAR_IRAM:				");
	asm("ld		[%r1]+,	%r0			");
	asm("cmp	%r1,	%r2			");
	asm("jrne   CLEAR_IRAM");

	// Clear Register
	asm("ld.a	%r1,	%r0			");
	asm("ld.a	%r2,	%r0			");
	asm("ld.a	%r3,	%r0			");
	asm("ld.a	%r4,	%r0			");
	asm("ld.a	%r5,	%r0			");
	asm("ld.a	%r6,	%r0			");
	asm("ld.a	%r7,	%r0			");

	// Set Stack Pointer
	asm("xld.a	%r0,	0x001000	");
	asm("ld.a	%sp,	%r0			");

	// Init. peripheral circuit
	FLSHC_WAIT = 0x00001;

	// Disable write protect
	CMU_PROTECT = 0x96;
	CMU_GATEDCLK0 |= 0x30;
	// Enable write protect
	CMU_PROTECT = 0x00;

	// Disable interrupt
	ITC_AITEN	= 0x0000;
	ITC_ITEN	= 0x0000;
	// Clear interrupt factor
	ITC_AITFT	= 0xFFEF;
	ITC_ITFT	= 0xFFFF;
	// Enable interrupt function
	ITC_ITCTL	= 0x0001;

	// Init. ".bss" and ".data" section
	LoadDataSec();

	// Jump main function
	main();

	// End program
	while(1);

}

/***************************************************************************
	LoadDataSec

	description	: Load .data section
	argument	: None
	return: 	: None

	input		:
	output		:

	flag		: None
	global		: None
***************************************************************************/
static void LoadDataSec(void){

	extern unsigned long	__START_data;
	extern unsigned long	__END_data;
	extern unsigned long	__START_data_lma;
	unsigned char			*pSrc;
	unsigned char			*pDst;
	unsigned long			dataSize;
	unsigned long			i;

	pSrc		= (unsigned char *)&__START_data_lma;
	pDst		= (unsigned char *)&__START_data;
	dataSize	= (unsigned char *)&__END_data - (unsigned char *)&__START_data;

	// Load ".data" section
	for(i = 0 ; i < dataSize ; i++){
		*pDst	= *pSrc;
		pSrc++;
		pDst++;

	}

	return;

}

