/***************************************************************************
*
*	File name : main.c
*	Function  : Main function
*	Revision  :
*		07/31/2007	Nariai		First revision
*
****************************************************************************/

// Include File
#include	"reg_801.h"
#include	"system.h"
#include	"usb_api.h"
#include	"descriptor.h"
#include	"classrequest.h"
#include	"vendorclass.h"


// Prototype
int main(void);
#ifndef	FOR_SNOOZE_OFF
void SnoozeUsb(void);
#endif
void ActiveUsb(void);
void USBIntEnable(void);
void USBIntDisable(void);


// Global

/***************************************************************************
	main

	description	: main function
	argument	:
	return: 	:

	input		:
	output		:

	flag		:
	global		:
***************************************************************************/
int main(void){

	unsigned short	dataNum;
	long			i;
	T_USB_CFG		config;
	unsigned char	*p;
	unsigned char	temp;

	// address/data
	EGPIO_P6_03_CFP = 0x55;
	EGPIO_P6_47_CFP = 0x55;
	EGPIO_P7_03_CFP = 0x55;
	EGPIO_P7_47_CFP = 0x55;
	EGPIO_P8_03_CFP = 0x55;
	EGPIO_P8_46_CFP = 0x55;
	EGPIO_P9_03_CFP = 0x55;
	EGPIO_P9_47_CFP = 0x55;
	EGPIO_PA_46_CFP = 0x55;
	EGPIO_PB_03_CFP = 0x55;
	EGPIO_PB_47_CFP = 0x55;

	// Select CE0
	EGPIO_PA_03_CFP = (EGPIO_PA_03_CFP & ~(EGPIO_PA_03_CFP_CFPA1 | EGPIO_PA_03_CFP_CFPA0)) | 0x05;

	SRAMC_EMSW = 0xFF22;
	SRAMC_EMSZ |= 0x0003;
	SRAMC_EMTS |= 0x0003;

	// set CMU
	// Disable write protect
	CMU_PROTECT		= 0x96;
	CMU_USBWT = (CMU_USBWT & ~CMU_USBWT_USBWT) | 0x05;
	// Enable write protect
	CMU_PROTECT		= 0x00;


	temp = (USB_USBStatus & USB_USBStatus_VBUS);
	// VBUS-check
	for(i = 0; i < 5; i++){
		if(temp != (USB_USBStatus & USB_USBStatus_VBUS)){
			temp = (USB_USBStatus & USB_USBStatus_VBUS);
			i = 0;
		}
	}

	if(temp != USB_USBStatus_VBUS){
		if(*(unsigned long *)0x100000 != 0xFFFFFFFF && *(unsigned long *)0x100000 != 0){
			// Disable write protect
			CMU_PROTECT = 0x96;
			CMU_GATEDCLK0 &= 0xCF;		// USB Clock off
			// Enable write protect
			CMU_PROTECT = 0x00;

			asm("xld.a	%r4,0x100000	");
			asm("ld.a	%r0,[%r4]		");
			asm("jpa	%r0				");
			while(1);
		}
	}

	// Init. ITC for USB
	ITC_AITEN &= ~ITC_AITEN_AIEN11;					// disable USB int.
	ITC_AITFT |= ITC_AITFT_AIFT11;					// clear ADC factor
	ITC_ATILV5 = (ITC_ATILV5 & 0x00FF) | 0x0700;	// interrupt level
	ITC_AITEN |= ITC_AITEN_AIEN11;					// enable USB int.

	// CPU interrupt enable
	CpuIntEnable();

	p = (unsigned char *)&config;
	for(i = 0; i < sizeof(T_USB_CFG); i++){
		*p++ = 0;
	}

	config.descriptorData = (unsigned char *)DescriptorData;	// Descriptor data pointer
	config.descriptorSize = DescriptorSize;						// Descriptor size

	config.activeFunc = ActiveUsb;			// Active clock function
#ifndef	FOR_SNOOZE_OFF
	config.snoozeFunc = SnoozeUsb;			// Snooze clock function
#endif
#ifdef	FOR_REMOTEWAKEUP
	config.remoteFunc = RemoteFunc;			// RemoteWakeup Switch function
#endif
#ifdef	FOR_CLASS_REQUEST
	config.resetFunc  = ResetFunc;			// Reset process function
	config.classFunc  = ClassFunc;			// Class request function
#endif
	config.vendorFunc = VendorFunc;			// Vendor request function
	config.intEnableFunc = USBIntEnable;	// USB Interrupt enable function
	config.intDisableFunc = USBIntDisable;	// USB Interrupt disable function

	if(UsbInit(&config) == ERROR){
		while(1);
	}
	VendorInit();

	while(1){
		UsbCheckInterrupt();
		UsbBusManagement();
		VendorMain();
	}

	return 0;
}

#ifndef	FOR_SNOOZE_OFF
/***************************************************************************
	SnoozeUsb

	description	: USB Snooze
	argument	: None
	return: 	: None

***************************************************************************/
void SnoozeUsb(void){

	// Disable write protect
	CMU_PROTECT = 0x96;
	CMU_USBWT |= CMU_USBWT_USBSNZ;
	// Enable write protect
	CMU_PROTECT = 0x00;

}
#endif

/***************************************************************************
	ActiveUsb

	description	: USB Active
	argument	: None
	return: 	: None

***************************************************************************/
void ActiveUsb(void){

	// Disable write protect
	CMU_PROTECT = 0x96;
	CMU_USBWT &= ~CMU_USBWT_USBSNZ;
	// Enable write protect
	CMU_PROTECT = 0x00;

}

/***************************************************************************
	USBIntEnable

	description	: Interrupt Enable for USB
	argument	: None
	return: 	: None

***************************************************************************/
void USBIntEnable(void){

	ITC_AITEN |= ITC_AITEN_AIEN11;	// enable USB int.
}


/***************************************************************************
	USBIntDisable

	description	: Interrupt Disable for USB
	argument	: None
	return: 	: None

***************************************************************************/
void USBIntDisable(void){

	ITC_AITEN &= ~ITC_AITEN_AIEN11;	// disable USB int.
}

