/***************************************************************************
*
*	File name : usb_api.h
*	Function  : USB driver (C17801)
*	Revision  :
*		August. 20, 2007	Yakushiji		First revision
*
****************************************************************************/

#ifndef	___USB_API_H___
#define	___USB_API_H___

// Request packet structure
typedef	struct {
	unsigned char		requestType;			// Request type
	unsigned char		request;				// Request number
	unsigned short		value;					// Value
	unsigned short		index;					// Index
	unsigned short		length;					// Length
} T_USB_REQUEST_PACKET;


// USB driver initialize configuration structure
typedef struct {
	unsigned char		*descriptorData;		// Descriptor data pointer
	unsigned short		descriptorSize;			// Descriptor size

	unsigned char		negotiationMode;		// Auto negotiation mode
												// USB_NEGOTIATION_AUTO
												// USB_NEGOTIATION_MANUAL

	unsigned char		addressMode;			// Auto address reply mode
												// USB_ADDRESS_AUTO
												// USB_ADDRESS_MANUAL

	unsigned char		managementMode;			// BUS processing mode of USB
												// USB_MANAGEMENT_AUTO
												// USB_MANAGEMENT_MANUAL

	void				(*activeFunc)(void);	// Active clock function
	void				(*snoozeFunc)(void);	// Snooze clock function
	void				(*resetFunc)(void);		// Reset process function
	int					(*remoteFunc)(void);	// RemoteWakeup Switch function
	int					(*classFunc)(T_USB_REQUEST_PACKET*);
												// Class request function
	int					(*vendorFunc)(T_USB_REQUEST_PACKET*);
												// Vendor request function
	void				(*intEnableFunc)(void);
												// USB Interrupt enable function
	void				(*intDisableFunc)(void);
												// USB Interrupt disable function
} T_USB_CFG;

// Management flag
typedef struct {
	unsigned char				Init:1;				// Initialize complete flag
	unsigned char				USBReqRcvd:1;		// Receive SetupPacket flag
	unsigned char				EnbRemoteWakeup:1;	// Enable RemoteWakeup flag
#ifdef	FOR_TEST_MODE
	unsigned char				TestMode:1;			// TestMode process flag
#endif
#ifdef	FOR_ADDRESS
	unsigned char				SetAddress:1;		// SetAddress request flag
#endif
	unsigned char				Attach:1;			// Attach request flag
	unsigned char				Detach:1;			// Detach request flag
	unsigned char				DetachProg:1;		// Detach Processing flag
	unsigned char				VbusChange:1;		// VbusChange request flag
	unsigned char				Reset:1;			// Reset request flag
	unsigned char				Suspend:1;			// Suspend request flag
	unsigned char				Resume:1;			// Resume request flag
	unsigned char				NonJ:1;				// NonJ flag
} T_MANAGEMENT_FLAG;


/***************************************************************
 *	UsbInit configuration parameter
 ***************************************************************/
#define	USB_NEGOTIATION_MANUAL		0		// Manual Negotiation
#define	USB_NEGOTIATION_AUTO		1		// Auto Negotiation

#define	USB_ADDRESS_MANUAL			0		// Manual address reply
#define	USB_ADDRESS_AUTO			1		// Auto address reply

#define	USB_MANAGEMENT_MANUAL		0		// Manual BUS management
#define	USB_MANAGEMENT_AUTO			1		// Auto BUS management

/***************************************************************
 *	Parameter for API function
 ***************************************************************/
#define	USB_VBUS_ON					USB_USBStatus_VBUS		// VBUS on
#define	USB_VBUS_OFF				0x00					// VBUS off

#define	USB_LINE_SE0				0x00					// SE0 state
#define	USB_LINE_J					0x01					// J state
#define	USB_LINE_K					0x02					// K state
#define	USB_LINE_SE1				0x03					// SE1 state

#define	USB_ENDPOINT_0				0						// Endpoint 0
#define	USB_ENDPOINT_A				1						// Endpoint a
#define	USB_ENDPOINT_B				2						// Endpoint b
#define	USB_ENDPOINT_C				3						// Endpoint c
#define	USB_ENDPOINT_D				4						// Endpoint d

#define	USB_TESTMODE_J				USB_USBTest_TestJ		// Test J mode
#define	USB_TESTMODE_K				USB_USBTest_TestK		// Test K mode
#define	USB_TESTMODE_SE0_NAK		USB_USBTest_TestSE0NAK	// Test SE0 NAK mode
#define	USB_TESTMODE_PACKET			USB_USBTest_TestPacket	// Test packet mode

#define	USB_AUTOADDRESS_ON			USB_USBAddress_AutoSetAddress	// Auto Address ON
#define	USB_AUTOADDRESS_OFF			0								// Auto Address OFF

#define	USB_OPMODE_NORMAL			0x00					// Normal
#define	USB_OPMODE_NONDRIVE			0x01					// Non-Drive
#define	USB_OPMODE_DISBITSTUFF		0x02					// Disable Bitstuffing and NRZI encoding
#define	USB_OPMODE_RESERVED			0x03

#define	USB_EP0_IN					USB_EP0Control_INxOUT	// IN
#define	USB_EP0_OUT					0						// OUT

#define	USB_EP_IN					0x80					// IN
#define	USB_EP_OUT					0x00					// OUT

#define	USB_EP_ISO					0x80					// Isochronous mode
#define	USB_EP_ISO_CRCMODE			0x40					// Isochronous CRC mode
#define	USB_EP_BULK_INTR			0x00					// Bulk or Interrupt

#define	USB_EP_TOGGLE_NORMAL		0x00					// Normal toggle mode
#define	USB_EP_TOGGLE_ALWAYS		0x40					// Always toggle mode

#define	USB_EP_CTL_STALL			0x01					// Force STALL
#define	USB_EP_CTL_NAK				0x02					// Force NAK
#define	USB_EP_CTL_TOGGLE_CLR		0x04					// Toggle clear
#define	USB_EP_CTL_TOGGLE_SET		0x08					// Toggle set
#define	USB_EP_CTL_DISAFNAK			0x20					// Disable Auto Force NAK when Short packet
#define	USB_EP_CTL_ENSHORTPKT		0x40					// Enable short packet
#define	USB_EP_CTL_AUTOFORCENAK		0x80					// Auto force NAK

#define	USB_EP_CTL_TOGGLE_0			0x00					// toggle state 0
#define	USB_EP_CTL_TOGGLE_1			0x10					// toggle state 1

// USB State
#define USB_DEFAULT_STATE			(0x00)
#define USB_ADDRESS_STATE			(0x01)
#define USB_CONFIGURED_STATE		(0x02)

#define	USB_INT_ENABLE				0x00
#define	USB_INT_DISABLE				0x01

/***************************************************************
 *	Interrupt factor
 ***************************************************************/
#define	USB_MAIN_INT_RCVSETUP		USB_MainIntEnb_EnRcvEP0SETUP	// EnRcvEP0SETUP
#define	USB_MAIN_INT_EP0			USB_MainIntEnb_EnEP0IntStat		// EnEP0IntStat
#define	USB_MAIN_INT_FIFO			USB_MainIntEnb_EnFIFOIntStat	// EnFIFOIntStat
#define	USB_MAIN_INT_EPR			USB_MainIntEnb_EnEPrIntStat		// EnEPrIntStat
#define	USB_MAIN_INT_SIE			USB_MainIntEnb_EnSIEIntStat		// EnSIEIntStat

#define	USB_SIE_INT_SETADDRESSCMP	USB_SIEIntEnb_EnSetAddressCmp	// EnSetAddressCmp
#define	USB_SIE_INT_DETECTJ			USB_SIEIntEnb_EnDetectJ			// EnDetectJ
#define	USB_SIE_INT_RCVSOF			USB_SIEIntEnb_EnRcvSOF			// EnRcvSOF
#define	USB_SIE_INT_DETECTSUSPEND	USB_SIEIntEnb_EnDetectSuspend	// EnDetectSuspend
#define	USB_SIE_INT_DETECTRESET		USB_SIEIntEnb_EnDetectReset		// EnDetectReset
#define	USB_SIE_INT_NONJ			USB_SIEIntEnb_EnNonJ			// EnNonJ
#define	USB_SIE_INT_VBUS			USB_SIEIntEnb_EnVBUSChanged		// EnVBUSChanged

#define	USB_EPR_INT_EPA				USB_EPrIntEnb_EnEPaIntStat		// EnEPaIntStat
#define	USB_EPR_INT_EPB				USB_EPrIntEnb_EnEPbIntStat		// EnEPbIntStat
#define	USB_EPR_INT_EPC				USB_EPrIntEnb_EnEPcIntStat		// EnEPcIntStat
#define	USB_EPR_INT_EPD				USB_EPrIntEnb_EnEPdIntStat		// EnEPdIntStat

#define	USB_FIFO_INT_DESCCMP		USB_FIFOIntStat_DescriptorCmp	// FIFO Interrupt Descriptor complete

#define	USB_EP_INT_OUT_TRAN_ERR		0x01							// OUT tansaction error
#define	USB_EP_INT_IN_TRAN_ERR		0x02							// IN tansaction error
#define	USB_EP_INT_OUT_TRAN_NAK		0x04							// Out tansaction NAK
#define	USB_EP_INT_IN_TRAN_NAK		0x08							// IN tansaction NAK
#define	USB_EP_INT_OUT_TRAN_ACK		0x10							// Out tansaction ACK
#define	USB_EP_INT_IN_TRAN_ACK		0x20							// IN tansaction ACK
#define	USB_EP_INT_OUT_SHORT_ACK	0x40							// Out Short ACK

/***************************************************************
 *	State
 ***************************************************************/
// Endpoint X pipe state
#define USB_EPX_DATA_SET			(0x00)
#define USB_EPX_DATA_SEND			(0x01)
#define USB_EPX_DATA_RCV			(0x02)
#define USB_EPX_DATA_GET			(0x03)
#define USB_EPX_IDLE				(0x04)
#define USB_EPX_STALL				(0x10)
#define USB_EPX_RESET				(0x20)
#define USB_EPX_FEATURE				(0x44)

/***************************************************************
 *	Result
 ***************************************************************/
#define	USB_EPX_NO_RESULT			(0)
#define	USB_EPX_IN_ERR				(1)
#define	USB_EPX_OUT_ERR				(2)
#define	USB_EPX_IN_ACK				(3)
#define	USB_EPX_OUT_ACK				(4)

/***************************************************************
 *	Retry Count
 ***************************************************************/
#define	USB_MAX_RETRY_CNT			(5)


/***************************************************************
 *	Macro function
 ***************************************************************/
#define	UsbFIFOAllClear()		(USB_EPnControl |= USB_EPnControl_AllFIFOClr)
#define	UsbFIFOIntEnable(x)		(USB_FIFOIntEnb |= x)
#define	UsbFIFOIntDisable(x)	(USB_FIFOIntEnb &= ~x)

#define	UsbGetAddress()			(USB_USBAddress & USB_USBAddress_USBAddress)
#define	UsbSetAddress(x)		(USB_USBAddress = (USB_USBAddress & ~USB_USBAddress_AutoSetAddress) | \
									(x & USB_USBAddress_USBAddress))
#define	UsbGetOpMode()			(USB_XcvrControl & USB_XcvrControl_OpMode)

#define	UsbGetEP0Direction()	(USB_EP0Control & USB_EP0Control_INxOUT)
#define	UsbSetEP0Direction(x)	(USB_EP0Control = (USB_EP0Control & ~USB_EP0Control_INxOUT) | x)

#define	UsbMainIntEnable(x)		(USB_MainIntEnb |= x)
#define	UsbMainIntDisable(x)	(USB_MainIntEnb &= ~x)
#define	UsbSIEIntEnable(x)		(USB_SIEIntEnb |= x)
#define	UsbSIEIntDisable(x)		(USB_SIEIntEnb &= ~x)
#define	UsbEPrIntEnable(x)		(USB_EPrIntEnb |= x)
#define	UsbEPrIntDisable(x)		(USB_EPrIntEnb &= ~x)

#define	UsbGetMainInt()			(USB_MainIntStat)
#define	UsbClearMainInt(x)		(USB_MainIntStat = x)
#define	UsbGetSIEInt()			(USB_SIEIntStat)
#define	UsbClearSIEInt(x)		(USB_SIEIntStat = x)
#define	UsbGetEPrInt()			(USB_EPrIntStat)
#define	UsbClearEPrInt(x)		(USB_EPrIntStat = x)
#define	UsbGetFIFOInt()			(USB_FIFOIntStat)
#define	UsbClearFIFOInt(x)		(USB_FIFOIntStat = x)
#define	UsbGetEP0Int()			(USB_EP0IntStat)
#define	UsbClearEP0Int(x)		(USB_EP0IntStat = x)
#define	UsbGetEPaInt()			(USB_EPaIntStat)
#define	UsbClearEPaInt(x)		(USB_EPaIntStat = x)
#define	UsbGetEPbInt()			(USB_EPbIntStat)
#define	UsbClearEPbInt(x)		(USB_EPbIntStat = x)
#define	UsbGetEPcInt()			(USB_EPcIntStat)
#define	UsbClearEPcInt(x)		(USB_EPcIntStat = x)
#define	UsbGetEPdInt()			(USB_EPdIntStat)
#define	UsbClearEPdInt(x)		(USB_EPdIntStat = x)
#define	Panic()					while(1)


#ifdef __cplusplus
extern "C" {
#endif

extern T_USB_CFG				g_Config;			// save initialize config data

// Interrupt factor
extern volatile unsigned char	g_MainInterruptFactor;
extern volatile unsigned char	g_SIEInterruptFactor;
extern volatile unsigned char	g_EPrInterruptFactor;
extern volatile unsigned char	g_FIFOInterruptFactor;
extern volatile unsigned char	g_EP0InterruptFactor;
extern volatile unsigned char	g_EPaInterruptFactor;
extern volatile unsigned char	g_EPbInterruptFactor;
extern volatile unsigned char	g_EPcInterruptFactor;
extern volatile unsigned char	g_EPdInterruptFactor;

extern unsigned short			g_UsbFifoMap[][6];		// FIFO area map

extern unsigned short			g_DeviceDescPoint;		// Device descriptor pointer
extern unsigned short			g_ConfigDescPoint[];	// Config descriptor pointer
extern unsigned short			g_InterfaceDescPoint[];	// Interface descriptor pointer
extern unsigned short			g_EndpointDescPoint;	// Endpoint descriptor pointer
extern unsigned short			g_StringDescPoint;		// String descriptor pointer

extern unsigned char			g_RemoteWakeupSupport;	// RemoteWakeup support flag
extern unsigned short			g_MaxPacket[][5];		// MaxPacket
extern unsigned char			g_EndpointNum[][4];		// Endpoint number
extern unsigned char			g_Attributes[][4];		// Endpoint Attribute
extern unsigned short			g_DescTotalLength;		// Descriptor total length

//extern unsigned char			g_EPChange[];

extern volatile T_MANAGEMENT_FLAG	g_ManagementFlag;	// Management control flag


// Low level API
extern unsigned char	UsbVBUSCheck(void);
extern unsigned char	UsbLineStateCheck(void);
extern int				UsbAttach(void);
extern void				UsbDetach(void);
extern void				UsbReset(void);
extern void				UsbSuspend(void);
extern void				UsbResume(void);
#ifdef	FOR_REMOTEWAKEUP
extern void				UsbRemoteWakeup(void);
#endif
extern int				UsbFIFORead(unsigned char, unsigned char *, unsigned short, unsigned short *);
#ifdef	FOR_BULK_TRANSFER
extern int				UsbFIFOWrite(unsigned char, unsigned char *, unsigned short, unsigned short *);
#endif
extern int				UsbFIFOReadRemain(unsigned char, unsigned short *);
extern int				UsbFIFOWriteRemain(unsigned char, unsigned short *);
extern int				UsbFIFOClear(unsigned char);
#ifndef	FOR_USB_MINIMUM_API
extern unsigned char	UsbGetFIFOReadJoin(void);
extern unsigned char	UsbGetFIFOWriteJoin(void);
#endif
#ifdef	FOR_TEST_MODE
extern int				UsbSetTestMode(unsigned char);
extern void				UsbStartTestMode(void);
#endif
extern void				UsbSetAutoAddress(unsigned char);
#ifndef	FOR_USB_MINIMUM_API
extern int				UsbSetOpMode(unsigned char);
#endif
extern void				UsbGetSetupPacket(T_USB_REQUEST_PACKET *);
#ifdef	FOR_BULK_TRANSFER
extern void				UsbEnableEndpoint(unsigned char);
extern void				UsbDisableEndpoint(unsigned char);
#endif
extern int				UsbSetEP0Control(unsigned char, unsigned char, unsigned char);
extern unsigned char	UsbGetEP0Control(unsigned char);
#ifdef	FOR_BULK_TRANSFER
extern int				UsbSetEndpointControl(unsigned char, unsigned char, unsigned char);
extern unsigned char	UsbGetEndpointControl(unsigned char);
#endif
extern int				UsbEndpointIntEnable(unsigned char, unsigned char);
extern int				UsbEndpointIntDisable(unsigned char, unsigned char);
#ifndef	FOR_USB_MINIMUM_API
extern unsigned char	UsbGetEndpointInt(unsigned char);
extern void				UsbClearEndpointInt(unsigned char, unsigned char);
#endif
extern void				UsbSetEndpointMaxSize(unsigned char, unsigned short);
#ifndef	FOR_USB_MINIMUM_API
extern unsigned short	UsbGetEndpointMaxSize(unsigned char);
#endif
#ifdef	FOR_BULK_TRANSFER
extern void				UsbSetEndpointNumber(unsigned char, unsigned char);
extern unsigned char	UsbGetEndpointNumber(unsigned char);
extern void				UsbSetEndpointDirection(unsigned char, unsigned char);
extern unsigned char	UsbGetEndpointDirection(unsigned char);
extern void				UsbSetEndpointMode(unsigned char, unsigned char);
extern unsigned char	UsbGetEndpointMode(unsigned char);
extern void				UsbSetEndpointToggleMode(unsigned char, unsigned char);
extern unsigned char	UsbGetEndpointToggleMode(unsigned char);
#endif
extern void				UsbSetFIFOArea(unsigned short *);
extern void				UsbGetFIFOArea(unsigned short *);
extern void				UsbWriteDescriptor(unsigned char *, unsigned short, unsigned short);
#ifndef	FOR_USB_MINIMUM_API
extern void				UsbReadDescriptor(unsigned char *, unsigned short, unsigned short);
#endif
extern void				UsbReplyDescriptor(unsigned short, unsigned short);
extern unsigned short	UsbRemainDescriptor(void);
extern unsigned char	UsbGetUSBState(void);


// High level API
extern void				UsbCtrlProcess(void);
extern void				UsbSetCtrlParameter(unsigned char *, unsigned short);
extern int				UsbCtrlRemainData(unsigned short *);
extern void				UsbCtrlFinish(int);

#ifdef	FOR_BULK_TRANSFER
extern void				UsbSetEPxParameter(unsigned char, unsigned char *, unsigned long);
extern unsigned long	UsbEPxRemainData(unsigned char);
extern void				UsbEPxSetState(unsigned char, unsigned char);
extern unsigned char	UsbEPxGetState(unsigned char);
extern void				UsbEPxClearResult(unsigned char);
extern unsigned char	UsbEPxGetResult(unsigned char);
extern void				UsbEPxDataSet(unsigned char);
extern void				UsbEPxDataGet(unsigned char);
#endif

extern int				UsbInit(T_USB_CFG *);
extern void				UsbBusManagement(void);
extern void				UsbInterruptHandler(void);
extern void				UsbCheckInterrupt(void);


#ifdef __cplusplus
}
#endif

#endif	/* __USB_API_H__	*/

