/***************************************************************************
*
*	File name : vendorclass.c
*	Function  : Vendor class sample application for C17801.
*	Revision  :
*		Aug/29/2007	Yakushiji		First revision
*
****************************************************************************/

// Include File
#include	"reg_801.h"
#include	"system.h"
#include	"usb_api.h"
#include	"descriptor.h"
#include	"flashc_api.h"
#include	"extflsc_api.h"

// Vendor request
#define GET_REWRITER_INFO	(0xFF)
#define END_DOWNLOAD		(0xFB)
#define WRITE_ROM			(0xFE)
#define READ_ROM			(0xFC)
#define FLASH_ERASE			(0xFD)


#define	RECIPIENT_DEVICE	(0x00)

static struct {
	unsigned long	startAddress;
	unsigned long	endAddress;
} EraseParameter;


unsigned char	ExitDownloader;


static unsigned long	RequestLength;
static unsigned char	WorkBuffer[64];
static unsigned char	Request;
static unsigned long	WriteRomAdd;

void VendorInit(void);
void VendorMain(void);
int VendorFunc(T_USB_REQUEST_PACKET *);

void UCODECopy(void);
void UCODEClear(void);

/*************************************************************
	VendorInit()

	description:
		Initialize of Vendor class.

	argument	: None
	return		: None

**************************************************************/
void VendorInit(void){

	RequestLength = 0;
	Request = 0;
	WriteRomAdd = 0;
}

/*************************************************************
	VendorMain()

	description:
		main process of Vendor class.

	argument	: None
	return		: None

**************************************************************/
void VendorMain(void){

	unsigned short	remain;
	unsigned short	size;
	int				ret;
	T_EXTFLSC_CFG	extConfig;

	if(ExitDownloader == 1 && UsbVBUSCheck() == USB_VBUS_OFF){
		if(*(unsigned long *)0x100000 != 0xFFFFFFFF && *(unsigned long *)0x100000 != 0){
			// Disable write protect
			CMU_PROTECT = 0x96;
			CMU_GATEDCLK0 &= 0xCF;		// USB Clock off
			// Enable write protect
			CMU_PROTECT = 0x00;

			asm("xld.a	%r4,0x100000	");
			asm("ld.a	%r4,[%r4]		");
			asm("jpa	%r4				");
			while(1);
		}
	}

	if(Request == WRITE_ROM || Request == FLASH_ERASE){

		if(UsbCtrlRemainData(&remain) == SUCCESS){
			if(remain == 0){
				switch(Request){
				case WRITE_ROM:

					if(RequestLength < sizeof(WorkBuffer)){
						size = RequestLength;
					} else {
						size = sizeof(WorkBuffer);
					}

					if(WriteRomAdd >= FLASHC_TOP_ADDRESS && WriteRomAdd <= FLASHC_END_ADDRESS){
						if(WriteRomAdd >= (FLASHC_TOP_ADDRESS + 0x2800) && WriteRomAdd <= FLASHC_END_ADDRESS){
							ret = FlashcWrite(WriteRomAdd, size, (unsigned short *)WorkBuffer);
						} else {
							ret = ERROR;
						}
					} else {
						ret = ExtFlscWrite(WriteRomAdd, size, (unsigned short *)WorkBuffer);
					}


					if(ret == ERROR){
						UCODEClear();

						UsbCtrlFinish(ret);
						RequestLength = 0;
						Request = 0;
						WriteRomAdd = 0;
					} else {
						RequestLength -= size;
						WriteRomAdd += size;

						if(RequestLength == 0){

							UCODEClear();

							UsbCtrlFinish(ret);
						} else {
							if(RequestLength < sizeof(WorkBuffer)){
								size = RequestLength;
							} else {
								size = sizeof(WorkBuffer);
							}

							UsbSetCtrlParameter(WorkBuffer, size);
						}
					}

					break;
				case FLASH_ERASE:
					UCODECopy();

					if((EraseParameter.startAddress >= FLASHC_TOP_ADDRESS &&
					    EraseParameter.startAddress <= FLASHC_END_ADDRESS) &&
					   (EraseParameter.endAddress >= FLASHC_TOP_ADDRESS &&
					    EraseParameter.endAddress <= FLASHC_END_ADDRESS)){

						if(EraseParameter.startAddress >= (FLASHC_TOP_ADDRESS + 0x2800) &&
						   EraseParameter.endAddress >= (FLASHC_TOP_ADDRESS + 0x2800)){
							ret = FlashcErase(FLASHC_ERASE_SECTOR, EraseParameter.startAddress,
										   EraseParameter.endAddress);
						} else {
							ret = ERROR;
						}
					} else {
						extConfig.control = EXTFLSC_TOP_ADDRESS;
						ExtFlscInit(&extConfig);

						ret = ExtFlscErase(EXTFLSC_ERASE_SECTOR, EraseParameter.startAddress,
									   EraseParameter.endAddress);
					}

					UCODEClear();

					UsbCtrlFinish(ret);

					RequestLength = 0;
					Request = 0;
					WriteRomAdd = 0;

					break;
				}
			}
		}
	}
}

/*************************************************************
	VendorFunc()

	description:
		Request function of Vendor class.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error complete

**************************************************************/
int VendorFunc(T_USB_REQUEST_PACKET *request){

	int				ret = ERROR;
	unsigned char	*p;
	int				i;
	unsigned short	size;
	unsigned long	ReadRomAdd;

	Request = request->request;
	RequestLength = request->length;

	if((request->requestType & 0x1F) != RECIPIENT_DEVICE){
		UsbCtrlFinish(ERROR);
		return ERROR;
	}

	if(UsbGetUSBState() != USB_CONFIGURED_STATE){
		UsbCtrlFinish(ERROR);
		return ERROR;
	}


	switch(request->request){
	case GET_REWRITER_INFO:

		if((request->requestType & 0x80) != 0x80){
			break;
		}

		p = (unsigned char *)&DescriptorData[0];
		i = 0;

		// search String Descriptor Index2
		while(1){
			if(*(p + 1) == 0x03){
				i++;
				if(i == 3){
					break;
				}
			}

			size = *p;
			p = p + size;
		}

		// check length
		if(request->length != (*p - 2)){
			break;
		}

		UsbSetCtrlParameter((p + 2), request->length);

		ret = SUCCESS;
		break;
	case END_DOWNLOAD:
		if((request->requestType & 0x80) != 0x00){
			break;
		}

		if(request->value || request->index || request->length){
			break;
		}

		// Check firmware write complete
		if(*(unsigned long *)(0x100000) != 0xFFFFFFFF){

			ExitDownloader = 1;

		} else {
			break;
		}

		ret = SUCCESS;
		break;
	case WRITE_ROM:
		if((request->requestType & 0x80) != 0x00){
			break;
		}

		// Get Write address
		((unsigned short *)&WriteRomAdd)[0] = request->value;
		((unsigned short *)&WriteRomAdd)[1] = request->index;

		if(request->length <= sizeof(WorkBuffer)){
			size = request->length;
		} else {
			size = sizeof(WorkBuffer);
		}

		UsbSetCtrlParameter(WorkBuffer, size);

		UCODECopy();

		ret = SUCCESS;
		return ret;
	case READ_ROM:
		if((request->requestType & 0x80) != 0x80){
			break;
		}

		// Get Read address
		((unsigned short *)&ReadRomAdd)[0] = request->value;
		((unsigned short *)&ReadRomAdd)[1] = request->index;

		UsbSetCtrlParameter((unsigned char *)ReadRomAdd, request->length);

		ret = SUCCESS;
		break;
	case FLASH_ERASE:
		if((request->requestType & 0x80) != 0x00){
			break;
		}

		if(request->value || request->index){	/* wValue */
			break;
		}

		UsbSetCtrlParameter((unsigned char *)&EraseParameter, request->length);

		ret = SUCCESS;
		return ret;
	default:
		break;
	}

	UsbCtrlFinish(ret);

	return ret;
}

void UCODECopy(void){

	extern unsigned long	__START_ucode_lma;
	extern unsigned long	__START_ucode;
	extern unsigned long	__END_ucode;
	extern unsigned long	__END_data;
	unsigned char			*pSrc;
	unsigned char			*pDst;
	unsigned long			dataSize;
	unsigned long			i;

	pSrc		= (unsigned char *)&__START_ucode_lma;
	pDst		= (unsigned char *)&__END_data;
	dataSize	= (unsigned char *)&__END_ucode - (unsigned char *)&__START_ucode;

	// Load ".data" section
	for(i = 0 ; i < dataSize ; i++){
		*pDst++	= *pSrc++;

	}

	return;
}

void UCODEClear(void){

	extern unsigned long	__START_ucode;
	extern unsigned long	__END_ucode;
	extern unsigned long	__END_data;
	unsigned char			*pDst;
	unsigned long			dataSize;
	unsigned long			i;

	pDst		= (unsigned char *)&__END_data;
	dataSize	= (unsigned char *)&__END_ucode - (unsigned char *)&__START_ucode;

	// Load ".data" section
	for(i = 0 ; i < dataSize ; i++){
		*pDst++ = 0;
	}

	return;
}
